/*	$Id: frame.c 2007/08/21 18:52:57 mallet $ */

/* 
 * Copyright (c) 2000-2007 LAAS/CNRS                  --  Wed May 10 2000
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ident "$Id: frame.c 2007/08/21 18:52:57 mallet $"

#ifdef VXWORKS
#include <vxWorks.h>
#endif

#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "frame.h"

int frameIds = 0;

/*
 * insituNewFrame -------------------------------------------------------
 *
 * Creates a new frame.
 */

InsituPvFrame *
insituNewFrame(char *name)
{
   InsituPvFrame *frame;

   if (strlen(name)>=ISFRM_MAX_LEN) { errno = EINVAL; return NULL; }

   frame = malloc(sizeof(InsituPvFrame));
   if (frame == NULL) { errno = ENOMEM; return NULL; }
   
   strcpy(frame->name, name);
   frame->id = -1;
   frame->path = NULL;
   frame->next = NULL;
   return frame;
}


/*
 * insituDeleteFrame ----------------------------------------------------
 *
 * Frees the frame and its name.
 */

void
insituDeleteFrame(InsituPvFrame *frame)
{
   if (frame->path != NULL) insituDeletePath(frame->path);
   free(frame);
}


/*
 * insituLookForFrame ---------------------------------------------------
 *
 * Looks for a frame in a list and returns it if it exists
 */

InsituPvFrame *
insituLookForFrame(char *name, InsituPvFrame *list)
{
   InsituPvFrame *f;

   for(f=list; f!=NULL; f=f->next)
      if (!strcmp(name, f->name)) return f;

   return NULL;
}


/*
 * insituGetFrameId -----------------------------------------------------
 *
 * Returns or create an id for this frame.
 */

int
insituGetFrameId(InsituPvFrame *frame)
{
   if (frame->id == -1) {
      frame->id = frameIds++;
   }

   return frame->id;
}


/*
 * insituGetFrameNameById -----------------------------------------------
 *
 */

char *
insituGetFrameNameById(InsituPvFrame *list, int id)
{
   for(; list!= NULL; list=list->next)
      if (list->id == id) return list->name;
   return "<unkown>";
}
