/*	$Id: geo.h 2007/08/21 19:12:08 mallet $ */

/*
 * Copyright (c) 2000-2007 LAAS/CNRS                  --  Fri May 12 2000
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef _GEO_H
#define _GEO_H

#include <sys/types.h>
#include <sys/stat.h>

#include "dynload.h"
#include "frame.h"
#include "graph.h"

typedef int (*InsituUpdateHook)(void);

typedef struct InsituUpdateHooks {
   char *name;			/* symbol name (insituWriteGeom only :( ) */
   InsituUpdateHook hook;	/* hook */
   struct InsituUpdateHooks *next;
} InsituUpdateHooks;

/* Geometrical config */
typedef struct InsituGeom {
   int refCount;		/* number of clients that use that
				 * description */

   ino_t ino;			/* inode number of the geometrical model
				 * file */
   dev_t dev;			/* ID of device containing a directory
				 * entry for the file */

   char *path;			/* path to file */
   char *file;			/* description file name */

   InsituPkg *pkgs;		/* list of dynamic packages */

   InsituPvFrame *frames;		/* list of frames (chained list) */
   int mainFrame;		/* id of the main frame */

   InsituUpdateHooks *updateHooks;
   InsituGraph *graph;		/* robot description */

   struct InsituGeom *next;
} InsituGeom;


/* Prototypes -------------------------------------------------------- */

InsituGeom *		insituNewGeom(char *geomFile);
void			insituDeleteGeom(InsituGeom *geom);
int			insituWriteGeom(InsituGeom *g, char *geomFile);
int			insituCleanUpGeom(InsituGeom *geom);

InsituUpdateHooks *	insituNewUpdateHook(char *name,
				InsituUpdateHook hook);


#endif /* _GEO_H */
