/*
 * Copyright (C) 2000 LAAS/CNRS 
 *
 * Anthony Mallet - Mon May 15 2000
 *
 * $Log$
 * Revision 1.1  2005/06/16 12:36:51  matthieu
 * Initial revision
 *
 * Revision 1.4  2000/07/24 16:41:53  mallet
 * utilisation de libt3d et petites corrections diverses
 *
 * Revision 1.3  2000/06/05 13:16:41  mallet
 * Adapte la fonction de sauvegarde du modle pour traiter correctement les
 * bibliothques partages ("require") et les callbacks ("update").
 *
 * Revision 1.2  2000/06/05 09:52:08  mallet
 * Ajoute la possibilit de dfinir des callbacks pour les liens dynamiques,
 * dans le but de rendre compltement robot-indpendant le module insitu.
 *
 * Un callback se definit  l'aide du mot cl "update"  l'intrieur de la
 * dfinition d'un lien (dans le fichier de description du robot). Il est
 * galement possible de dfinir un callback  l'extrieur de tout lien. Ce
 * dernier sera invoqu avant la mise  jour de chaque lien (pour factoriser
 * le lecture de posters, par exemple, si un poster participe  plusieurs
 * liens).
 *
 * Un callcack est le nom d'une fonction C. Celle-ci doit tre accessible par
 * la bibliothque. Un mot clef "require" permet de spcifier le nom d'une
 * bibliothque partage qui sera charge au moment de la lecture du fichier
 * de description.
 *
 * A l'intrieur d'un lien, un callback doit tre de type InsituMatrix (et
 * retourner une matrice homogne 4x4 (16 doubles) contenant la nouvelle
 * configuration du lien).  A l'extrieur de tout lien, un callback doit tre
 * de type InsituUpdateHook (et retourner 0 en cas de succs, autre chose en
 * cas d'erreur).
 *
 * 	typedef double *(*InsituMatrix)(void);
 * 	typedef int (*InsituUpdateHook)(void);
 *
 * Revision 1.1  2000/05/25 14:15:58  mallet
 * Bibliothque des gestion des modles geomtriques pour les robots (insitu
 * pour "internal situation").
 *
 * Insitu permet,  partir d'un fichier de description ascii, de mettre  jour
 * les configurations des divers organes mobiles du modle du robot et
 * d'obtenir les matrices de passage des divers repres (dfinis dans le
 * fichier de configuration) vers un repre principal, commun  tous les
 * modules.
 *
 * Dans la spcification actuelle, Insitu est destin  tre encapsul dans un
 * module, rafrachissant priodiquement les configuration de tous les organes
 * mobiles et exportant les matrices de passages de tous les repres dans un
 * (ou des) posters. Les capteurs pourront ainsi accompagner leur donnes de
 * la matrice qui les concerne. La version calife est, comme d'habitude,
 * destine au traitement hors ligne de ces donnes, pour simulation ou
 * debugage.
 *
 * Insitu fournit sept fonctions (ce sont ici les fonctions C qui sont
 * dcrites. La version iwac est strictement similaire) :
 *
 *
 * InsituClientId	insituOpen(char *geomFile, InsituAccesMode mode);
 *
 *    Lit une description geometrique d'un robot, et retourne un objet
 *    permettant de manipuler cette description. Le mode d'accs
 *    (ISMODE_READONLY, ISMODE_READWRITE, ISMODE_EDIT) dfinit l'utilisation
 *    du modle. Voir la documentation de cette fonction dans le source.
 *
 *
 * InsituLinkId	insituOpenLink(InsituClientId id, char *from, char *to);
 *
 *    Ouvre un lien dans le modle (une connection directe entre deux
 *    repres), pour modification. En mode read/write, on ne peut modifier que
 *    les liens correspondant aux organe mobiles. En mode edit, on peut
 *    galement modifier les liens dit "statiques" (pour enregistrer le
 *    resultat d'une calibration par exemple).
 *
 *
 * InsituFrameId	insituOpenFrame(InsituClientId id, char *frame);
 *
 *    Ouvre un repre du modle. Celui-ci permet de lire la matrice de passage
 *    courante vers le repre central.
 *
 *
 * int		insituWrite(InsituClientId id, char *file);
 *
 *    Ecrit le modle (et les configurations courantes des liens statiques)
 *    dans un fichier. Utile si l'on a precdement ouvert un modle en mode
 *    edit.
 *
 *
 * void		insituClose(InsituClientId id);
 *
 *    Detruit un modle.
 *
 *
 * int	insituSet(InsituLinkId link, double *matrix);
 * int	insituGet(InsituFrameId frame, double *matrix);
 *
 *    Affecte une matrice  un lien (set, mode read/write ou edit) ou lit la
 *    position d'un repre relativement au repre central (get).
 *
 * Revision 1.4  2000/05/24 17:15:13  mallet
 * devel
 *
 * Revision 1.3  2000/05/15 18:29:17  mallet
 * devel
 *
 * 
 */
#ifndef _GEOM_H
#define _GEOM_H

#define Y_UNION(t, v) struct { int l; t v; } v

/* A temporary struct for links */
struct linkdecl {
   int flag;

   InsituPvFrame *from;
   InsituPvFrame *to;
   double translate[3];
   double rotate[3];
   InsituMatrix update;
   char *updateName;

#define ISLNKDCL_FROM		1
#define ISLNKDCL_TO		1<<1
#define ISLNKDCL_TRANSLATE	1<<2
#define ISLNKDCL_ROTATE		1<<3
#define ISLNKDCL_UPDATE		1<<4
};
 

#endif /* _GEOM_H */
