/*	$Id: graph.h 2007/08/21 18:18:32 mallet $ */

/* 
 * Copyright (c) 2000-2007 LAAS/CNRS                  --  Mon May 15 2000
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef _GRAPH_H
#define _GRAPH_H

#include "t3d/t3d.h"

/* Link structure ---------------------------------------------------- */

typedef enum InsituLinkType {
   ISLINK_STATIC,
   ISLINK_DYNAMIC
} InsituLinkType;

typedef T3D *(*InsituMatrix)(void);

typedef struct InsituLink {
   InsituLinkType type;
   T3D t3d;			/* matrix */
   T3D d3t;			/* inverse matrix */
   InsituMatrix update;		/* function that updates the link */
   char *updateName;		/* function name */
} InsituLink;


/* Graph structure --------------------------------------------------- */

typedef struct InsituGraph {
   int nodes;			/* number of nodes */
   InsituLink **node;		/* graph ("nodes" by "nodes" array) */
} InsituGraph;


/* Path inside the graph --------------------------------------------- */

typedef enum InsituLinkDir {
   ISLINK_FORWARD, ISLINK_BACKWARD
} InsituLinkDir;

typedef struct InsituPath {
   InsituLinkDir dir;		/* how to follow this link */
   InsituLink *link;		/* link */
   struct InsituPath *next;	/* next link to follow */
} InsituPath;


/* Prototypes -------------------------------------------------------- */

InsituGraph *	insituMakeLink(InsituGraph *graph, InsituLinkType type, 
			int a, int b);
int		insituTestValidLink(InsituGraph *graph, int a, int b);

void		insituSetLink(InsituGraph *graph, int a, int b, T3D *t3d);
void		insituSetInvLink(InsituGraph *graph, int a, int b, T3D *t3d);

void		insituSetLinkUpdate(InsituGraph *graph, int a, int b,
			char *name, InsituMatrix update);

int		insituUpdateGraph(InsituGraph *graph);

InsituPath *	insituGetPath(InsituGraph *graph, int a, int b);
void		insituDeletePath(InsituPath *path);

void		insituDeleteGraph(InsituGraph *graph);

#endif /* _GRAPH_H */
